@@ -35,13 +35,9 @@
 		stop("primer is not the same length as target.")
 	
 	if (align) {
-		p <- pairwiseAlignment(primer,
-			target,
-			type="global",
-			gapOpen=-5,
-			gapExtension=-5)
-		primer <- as.character(pattern(p))
-		target <- as.character(subject(p))
+		p <- .pairwiseAlignment(primer, target, TRUE)
+		primer <- p$pattern
+		target <- p$subject
 	} else {
 		if (any(nchar(target) != nchar(primer)))
 			stop("primer and target must be aligned (equal length).")
@@ -55,7 +51,7 @@
 		if (!is.null(processors) && processors < 1)
 			stop("processors must be at least 1.")
 		if (is.null(processors)) {
-			processors <- detectCores()
+			processors <- .detectCores()
 		} else {
 			processors <- as.integer(processors)
 		}
@@ -240,7 +236,7 @@ DesignSignatures <- function(dbFile,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- detectCores()
+		processors <- .detectCores()
 	} else {
 		processors <- as.integer(processors)
 	}
@@ -301,20 +297,21 @@ DesignSignatures <- function(dbFile,
 	}
 	
 	# initialize database
-	driver = dbDriver("SQLite")
 	if (is.character(dbFile)) {
-		dbConn = dbConnect(driver, dbFile)
+		if (!requireNamespace("RSQLite", quietly=TRUE))
+			stop("Package 'RSQLite' must be installed.")
+		dbConn <- dbConnect(dbDriver("SQLite"), dbFile)
 		on.exit(dbDisconnect(dbConn))
 	} else {
-		dbConn = dbFile
-		if (!inherits(dbConn,"SQLiteConnection")) 
-			stop("'dbFile' must be a character string or SQLiteConnection.")
+		dbConn <- dbFile
 		if (!dbIsValid(dbConn))
 			stop("The connection has expired.")
 	}
 	
-	searchExpression <- paste("select distinct identifier from",
-		tblName)
+	searchExpression <- paste("select distinct",
+		dbQuoteIdentifier(dbConn, "identifier"),
+		"from",
+		dbQuoteIdentifier(dbConn, tblName))
 	rs <- dbSendQuery(dbConn, searchExpression)
 	searchResult <- dbFetch(rs, n=-1, row.names=FALSE)
 	ids <- searchResult$identifier
@@ -335,11 +332,6 @@ DesignSignatures <- function(dbFile,
 			stop("focusID not found in identifier.")
 	}
 	
-	# set the seed for repeatable sampling
-	last.seed <- .Random.seed
-	set.seed(1234)
-	on.exit({.Random.seed <- last.seed}, add=TRUE)
-	
 	# load dS and dH rules
 	data("deltaSrules",
 		"deltaHrules",
@@ -2060,6 +2052,7 @@ DesignSignatures <- function(dbFile,
 			time.1,
 			units='secs'),
 			digits=2))
+		cat("\n")
 	}
 	
 	return(primers)
