@@ -44,7 +44,7 @@ FindNonCoding <- function(x,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- detectCores()
+		processors <- .detectCores()
 	} else {
 		processors <- as.integer(processors)
 	}
@@ -135,6 +135,9 @@ FindNonCoding <- function(x,
 				DNAStringSet(myXString),
 				K[[k]],
 				TRUE, # mask repeats
+				FALSE, # mask low complexity regions
+				integer(), # mask numerous k-mers
+				1L, # left is fast moving side
 				processors,
 				PACKAGE="DECIPHER")[[1L]]
 			
