@@ -33,20 +33,19 @@ IdConsensus <- function(dbFile,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- detectCores()
+		processors <- .detectCores()
 	} else {
 		processors <- as.integer(processors)
 	}
 	
 	# initialize database
-	driver = dbDriver("SQLite")
 	if (is.character(dbFile)) {
-		dbConn = dbConnect(driver, dbFile)
+		if (!requireNamespace("RSQLite", quietly=TRUE))
+			stop("Package 'RSQLite' must be installed.")
+		dbConn <- dbConnect(dbDriver("SQLite"), dbFile)
 		on.exit(dbDisconnect(dbConn))
 	} else {
-		dbConn = dbFile
-		if (!inherits(dbConn,"SQLiteConnection")) 
-			stop("'dbFile' must be a character string or SQLiteConnection.")
+		dbConn <- dbFile
 		if (!dbIsValid(dbConn))
 			stop("The connection has expired.")
 	}
@@ -57,13 +56,15 @@ IdConsensus <- function(dbFile,
 		stop(paste("The colName '", colName, "' does not exist.", sep=""))
 	
 	searchExpression <- paste('select distinct ',
-		colName,
+		dbQuoteIdentifier(dbConn, colName),
 		' from ',
-		tblName,
+		dbQuoteIdentifier(dbConn, tblName),
 		sep="")
 	if (identifier != "")
 		searchExpression <- paste(searchExpression,
-			" where identifier is '",
+			" where",
+			dbQuoteIdentifier(dbConn, "identifier"),
+			"= '",
 			identifier,
 			"'",
 			sep="")
