@@ -32,20 +32,19 @@ IdLengths <- function(dbFile,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- detectCores()
+		processors <- .detectCores()
 	} else {
 		processors <- as.integer(processors)
 	}
 	
 	# initialize database
-	driver <- dbDriver("SQLite")
 	if (is.character(dbFile)) {
-		dbConn <- dbConnect(driver, dbFile)
+		if (!requireNamespace("RSQLite", quietly=TRUE))
+			stop("Package 'RSQLite' must be installed.")
+		dbConn <- dbConnect(dbDriver("SQLite"), dbFile)
 		on.exit(dbDisconnect(dbConn))
 	} else {
 		dbConn <- dbFile
-		if (!inherits(dbConn,"SQLiteConnection")) 
-			stop("'dbFile' must be a character string or SQLiteConnection.")
 		if (!dbIsValid(dbConn))
 			stop("The connection has expired.")
 	}
@@ -85,9 +84,9 @@ IdLengths <- function(dbFile,
 			tblName=tblName,
 			type=TYPES[type],
 			replaceChar="+",
-			clause=paste("rowid > ",
+			clause=paste("row_names > ",
 				(i - 1)*batchSize,
-				" and rowid <= ",
+				" and row_names <= ",
 				i*batchSize,
 				sep=""),
 			processors=processors,
@@ -118,7 +117,7 @@ IdLengths <- function(dbFile,
 			verbose=FALSE)
 	
 	if (verbose) {
-		cat("\nLengths counted for ", count, " sequences.", sep="")
+		cat("\n\nLengths counted for ", count, " sequences.", sep="")
 		if (is.character(add2tbl) || add2tbl)
 			cat("\nAdded to ",
 				ifelse(is.character(add2tbl), add2tbl, tblName),
