@@ -111,7 +111,7 @@ IdTaxa <- function(test,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- detectCores()
+		processors <- .detectCores()
 	} else {
 		processors <- as.integer(processors)
 	}
@@ -168,6 +168,9 @@ IdTaxa <- function(test,
 			K,
 			trainingSet$alphabet,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			processors,
 			PACKAGE="DECIPHER")
 	} else {
@@ -175,6 +178,9 @@ IdTaxa <- function(test,
 			test,
 			K,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			processors,
 			PACKAGE="DECIPHER")
 	}
@@ -237,6 +243,9 @@ IdTaxa <- function(test,
 			reverseComplement(test[boths]),
 			K,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			processors,
 			PACKAGE="DECIPHER")
 		revkmers <- lapply(revkmers,
