@@ -179,6 +179,9 @@ LearnTaxa <- function(train,
 			K,
 			alphabet,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			1L, # processors
 			PACKAGE="DECIPHER")
 	} else {
@@ -186,6 +189,9 @@ LearnTaxa <- function(train,
 			train,
 			K,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			1L, # processors
 			PACKAGE="DECIPHER")
 	}
@@ -287,8 +293,13 @@ LearnTaxa <- function(train,
 				if (length(w) == 0)
 					stop("All values of Parent in rank must have a corresponding Index.")
 				if (rank$Name[w] != taxa[parents[i]])
-					stop("rank contains an unexpected Parent for Name: ",
-						taxa[i])
+					stop("rank contains an unexpected Parent for ",
+						taxa[i],
+						": ",
+						rank$Name[w],
+						" (expected ",
+						taxa[parents[i]],
+						")")
 			}
 		}
 	}
