@@ -1,7 +1,7 @@
 OrientNucleotides <- function(myXStringSet,
 	reference=which.max(width(myXStringSet)),
 	type="sequences",
-	orientation="all",
+	orientation="both",
 	threshold=0.05,
 	verbose=TRUE,
 	processors=1) {
@@ -51,7 +51,7 @@ OrientNucleotides <- function(myXStringSet,
 	if (!is.null(processors) && processors < 1)
 		stop("processors must be at least 1.")
 	if (is.null(processors)) {
-		processors <- detectCores()
+		processors <- .detectCores()
 	} else {
 		processors <- as.integer(processors)
 	}
@@ -98,6 +98,9 @@ OrientNucleotides <- function(myXStringSet,
 		myXStringSet,
 		wordSize,
 		FALSE, # mask repeats
+		FALSE, # mask low complexity regions
+		integer(), # mask numerous k-mers
+		1L, # left is fast moving side
 		processors,
 		PACKAGE="DECIPHER")
 	v <- lapply(v, sort)
@@ -129,6 +132,9 @@ OrientNucleotides <- function(myXStringSet,
 			seqs,
 			wordSize,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			processors,
 			PACKAGE="DECIPHER")
 		v <- lapply(v, sort.int)
@@ -159,6 +165,9 @@ OrientNucleotides <- function(myXStringSet,
 			seqs,
 			wordSize,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			processors,
 			PACKAGE="DECIPHER")
 		v <- lapply(v, sort.int)
@@ -189,6 +198,9 @@ OrientNucleotides <- function(myXStringSet,
 			seqs,
 			wordSize,
 			FALSE, # mask repeats
+			FALSE, # mask low complexity regions
+			integer(), # mask numerous k-mers
+			1L, # left is fast moving side
 			processors,
 			PACKAGE="DECIPHER")
 		v <- lapply(v, sort.int)
