@@ -2,6 +2,11 @@
  *                       Converts Sequence To Numbers                       *
  *                           Author: Erik Wright                            *
  ****************************************************************************/
+ 
+ // for OpenMP parallel processing
+ #ifdef _OPENMP
+ #include <omp.h>
+ #endif
 
 /*
  * Rdefines.h is needed for the SEXP typedef, for the error(), INTEGER(),
@@ -22,11 +27,6 @@
 /* for Calloc/Free */
 #include <R_ext/RS.h>
 
-// for OpenMP parallel processing
-#ifdef SUPPORT_OPENMP
-#include <omp.h>
-#endif
-
 // for math functions
 #include <math.h>
 
@@ -65,30 +65,52 @@ static void alphabetFrequency(const Chars_holder *P, int *bits, int position)
 }
 
 // changes repeat regions to NAs
-static void maskRepeats(int *x, int n, int l1, int l2, int l3, int l)
+static void maskRepeats(int *x, int n, int l1, int l2, int l3, int l4, int l, double prob)
 {
-	// n: size
+	// n: word size
 	// l1: min period
 	// l2: max period
 	// l3: min length of repeat
-	int i, p, j, k;
+	// l4: max positions between matches
+	// prob: expected probability of match
+	
+	int i, p, j, k, c;
+	double m, s, t;
 	
 	i = 0; // current position
-	while (i < (l - l2)) {
+	while (i < l) {
 		if (x[i] != NA_INTEGER) {
-			for (p = l1; p <= l2; p++) { // periodicity
+			for (p = l1; p <= l2 && i + p < l; p++) { // periodicity
 				if (x[i] == x[i + p]) { // repeat
-					j = i + 1;
-					
-					while (j < (l - p)) {
-						if (x[j] != x[j + p])
-							break;
-						j++;
+					m = 1; // number of matches
+					s = m - prob*p; // score = matches above expectation
+					j = i + 1; // last match
+					k = j; // position
+					c = 0; // mismatch count
+					while (k < l - p) {
+						if (x[k] == x[k + p]) {
+							m++; // increment match
+							t = m - prob*(k + p - i); // temp score
+							if (t <= 0) { // fewer matches than expected
+								break;
+							} else if (t > s) {
+								s = t; // new high score
+								j = ++k; // update last position
+							}
+							c = 0; // reset mismatch count
+						} else {
+							if (c >= l4) // too many mismatches
+								break;
+							c++; // increment mismatch count
+							k++;
+						}
 					}
 					
-					if ((j - i + n) > p && // continuous repeat
+					if (s > 0 && // higher score than expected
+						(j - i + n) > p && // continuous repeat
 						(j + p - i + n) > l3) {
-						for (k = i; k <= (j + p - 1); k++)
+						// mask all repeat units after the first unit
+						for (k = i + p; k <= (j + p - 1); k++)
 							x[k] = NA_INTEGER;
 						i = k - 1;
 						break;
@@ -100,11 +122,178 @@ static void maskRepeats(int *x, int n, int l1, int l2, int l3, int l)
 	}
 }
 
-SEXP enumerateSequence(SEXP x, SEXP wordSize, SEXP mask, SEXP nThreads)
+// changes low complexity regions to NAs
+static void maskSimple(int *x, int n, double *E, int l1, int l2, double l3, int l)
+{
+	// n: word size
+	// E: expected counts
+	// l1: number of bins
+	// l2: window size
+	// l3: threshold for statistical significance
+	
+	int j, k, sum;
+	double freq[l1]; // level frequencies
+	for (j = 0; j < l1; j++)
+		freq[j] = 0;
+	int s = 0; // sum of frequencies
+	int c = 0; // count of positions
+	int pos[n]; // store previous positions for masking
+	int prev[l2]; // previous values before masking
+	int curr = 0; // index in prev
+	
+	for (j = 0; j < l; j++) {
+		if (s == l2) { // remove position outside of window
+			sum = prev[curr];
+			freq[sum]--;
+			s--;
+		}
+		sum = *(x + j);
+		if (sum >= 0 && sum != NA_INTEGER) {
+			sum %= l1;
+			prev[curr++] = sum;
+			if (curr == l2)
+				curr = 0;
+			freq[sum]++;
+			s++;
+		} // else continue with previous frequencies
+		double score = 0;
+		double temp;
+		double expected;
+		for (k = 0; k < l1; k++) { // Pearson's chi-squared test
+			expected = E[k]*s;
+			temp = freq[k] - expected;
+			temp *= temp;
+			score += temp/expected;
+		}
+		if (score > l3) { // mask
+			if (c < n) {
+				pos[c] = j - s/2;
+				c++;
+			} else if (c == n) {
+				for (c = 0; c < n; c++)
+					*(x + pos[c]) = NA_INTEGER;
+				*(x + j - s/2) = NA_INTEGER;
+				c++;
+			} else { // c > n
+				*(x + j - s/2) = NA_INTEGER;
+			}
+		} else {
+			c = 0;
+		}
+	}
+	while (s > 1) {
+		curr--;
+		if (curr < 0)
+			curr = l2 - 1;
+		sum = prev[curr];
+		freq[sum]--;
+		
+		double score = 0;
+		double temp;
+		double expected;
+		for (k = 0; k < l1; k++) { // Pearson's chi-squared test
+			expected = E[k]*s;
+			temp = freq[k] - expected;
+			temp *= temp;
+			score += temp/expected;
+		}
+		if (score > l3) { // mask
+			if (c < n) {
+				pos[c] = j - s/2;
+				c++;
+			} else if (c == n) {
+				for (c = 0; c < n; c++)
+					*(x + pos[c]) = NA_INTEGER;
+				*(x + j - s/2) = NA_INTEGER;
+				c++;
+			} else { // c > n
+				*(x + j - s/2) = NA_INTEGER;
+			}
+		} else {
+			c = 0;
+		}
+		s--;
+	}
+}
+
+// changes k-mers that are too numerous to NAs
+static void maskNumerous(int *x, int n, int tot, int l, int wS)
+{
+	// n: mask k-mers more numerous
+	// tot: number of k-mers possible
+	// l: number of k-mers in x
+	// wS: word size (minimum mask length)
+	
+	int i;
+	unsigned int j, k, mod;
+	int maxCollisions = 100;
+	if (l < tot) {
+		mod = (unsigned int)l; // collisions possible
+	} else {
+		mod = (unsigned int)tot; // collisions impossible
+	}
+	int *counts = Calloc(mod, int); // initialized to zero
+	int *keys = Calloc(mod, int); // initialized to zero
+	
+	// count k-mers (assumes most frequent occur before maxCollisions)
+	for (i = 0; i < l; i++) {
+		if (x[i] != NA_INTEGER) {
+			k = 0;
+			do {
+				j = (unsigned int)x[i] + k*(k + 1)/2;
+				j %= mod;
+				if (counts[j] == 0) { // new key
+					counts[j] = 1;
+					keys[j] = x[i];
+					break;
+				} else if (x[i] == keys[j]) { // existing key
+					counts[j]++;
+					break;
+				} // else collision
+				k++;
+			} while (k < maxCollisions);
+		}
+	}
+	
+	// convert k-mers that are too numerous to NA
+	int count = 0;
+	for (i = 0; i < l; i++) {
+		if (x[i] == NA_INTEGER) { // already masked
+			count++;
+		} else {
+			k = 0;
+			do {
+				j = (unsigned int)x[i] + k*(k + 1)/2;
+				j %= mod;
+				if (x[i] == keys[j]) { // matching key
+					if (counts[j] > n) {
+						count++;
+						if (count == wS) {
+							for (j = 0; j < wS; j++)
+								x[i - j] = NA_INTEGER;
+						} else if (count > wS) {
+							x[i] = NA_INTEGER;
+						}
+					} else {
+						count = 0;
+					}
+					break;
+				}
+				k++;
+			} while (k < maxCollisions);
+		}
+	}
+	
+	Free(counts);
+	Free(keys);
+}
+
+SEXP enumerateSequence(SEXP x, SEXP wordSize, SEXP mask, SEXP maskLCRs, SEXP maskNum, SEXP fastMovingSide, SEXP nThreads)
 {
 	XStringSet_holder x_set;
 	Chars_holder x_i;
 	int x_length, i, j, k, wS, maskReps, sum, ambiguous, *rans;
+	int fast = asInteger(fastMovingSide);
 	int nthreads = asInteger(nThreads);
 	
 	// initialize the XStringSet
@@ -112,16 +301,41 @@ SEXP enumerateSequence(SEXP x, SEXP wordSize, SEXP mask, SEXP nThreads)
 	x_length = get_length_from_XStringSet_holder(&x_set);
 	wS = asInteger(wordSize); // [1 to 15]
 	maskReps = asInteger(mask);
+	double prob = pow(0.7, wS); // expected probability of match
+	double missed = -48.35429; // log(probability of no matches)
+	int maxMismatches = (int)(missed/log(1 - prob)); // interval between matches
+	
+	// initialize variables for masking low complexity regions
+	double threshold = 12.66667;
+	double threshold2 = 38.90749;
+	int maskSimp = (asInteger(maskLCRs) == 0) ? 0 : 20; // window size
+	int maskSimp2 = (asInteger(maskLCRs) == 0) ? 0 : 95; // window size
+	double E[4] = {0.25, 0.25, 0.25, 0.25}; // expected frequency
+	
+	// initialize variables for masking numerous k-mers
+	int l = length(maskNum);
+	int *mN;
+	int tot; // total number of k-mers
+	if (l > 0) {
+		mN = INTEGER(maskNum);
+		tot = 1;
+		for (i = 0; i < wS; i++)
+			tot *= 4;
+	}
 	
 	SEXP ret_list;
 	PROTECT(ret_list = allocVector(VECSXP, x_length));
 	
 	// fill the position weight vector
-	int pwv[wS]; // wS[0] is ignored
-	if (wS > 1)
-		pwv[1] = 4;
-	for (i = 2; i < wS; i++) {
-		pwv[i] = pwv[i - 1]*4;
+	int pwv[wS];
+	if (fast) { // left side moves faster
+		pwv[0] = 1;
+		for (i = 1; i < wS; i++)
+			pwv[i] = pwv[i - 1]*4;
+	} else { // right side moves faster
+		pwv[wS - 1] = 1;
+		for (i = wS - 2; i >= 0; i--)
+			pwv[i] = pwv[i + 1]*4;
 	}
 	
 	// build a vector of thread-safe pointers
@@ -140,7 +354,9 @@ SEXP enumerateSequence(SEXP x, SEXP wordSize, SEXP mask, SEXP nThreads)
 		UNPROTECT(1);
 	}
 	
+	#ifdef _OPENMP
 	#pragma omp parallel for private(i,j,k,x_i,rans,sum,ambiguous) num_threads(nthreads)
+	#endif
 	for (i = 0; i < x_length; i++) {
 		x_i = get_elt_from_XStringSet_holder(&x_set, i);
 		if ((x_i.length - wS + 1) >= 1) {
@@ -151,7 +367,7 @@ SEXP enumerateSequence(SEXP x, SEXP wordSize, SEXP mask, SEXP nThreads)
 			}
 			for (j = wS - 1; j < x_i.length; j++) {
 				alphabetFrequency(&x_i, &bases[wS - 1], j);
-				sum = bases[0];
+				sum = bases[0]*pwv[0];
 				ambiguous = 0;
 				if (bases[0] < 0)
 					ambiguous = 1;
@@ -169,7 +385,15 @@ SEXP enumerateSequence(SEXP x, SEXP wordSize, SEXP mask, SEXP nThreads)
 			}
 			
 			if (maskReps)
-				maskRepeats(rans, wS, 7, 12, 30, x_i.length - wS + 1);
+				maskRepeats(rans, wS, 1, 700, 25, maxMismatches, x_i.length - wS + 1, prob);
+			
+			if (maskSimp)
+				maskSimple(rans, wS, E, 4, maskSimp, threshold, x_i.length - wS + 1);
+			if (maskSimp2)
+				maskSimple(rans, wS, E, 4, maskSimp2, threshold2, x_i.length - wS + 1);
+			
+			if (l == x_length)
+				maskNumerous(rans, mN[i], tot, x_i.length - wS + 1, wS);
 		}
 	}
 	
@@ -636,12 +860,13 @@ static void alphabetFrequencyReducedAA(const Chars_holder *P, int *bits, int pos
 	}
 }
 
-SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask, SEXP nThreads)
+SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask, SEXP maskLCRs, SEXP maskNum, SEXP fastMovingSide, SEXP nThreads)
 {
 	XStringSet_holder x_set;
 	Chars_holder x_i;
 	int x_length, i, j, k, wS, maskReps, sum, ambiguous, *rans;
 	int *alpha = INTEGER(alphabet);
+	int fast = asInteger(fastMovingSide);
 	int nthreads = asInteger(nThreads);
 	
 	// initialize the XStringSet
@@ -649,6 +874,9 @@ SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask,
 	x_length = get_length_from_XStringSet_holder(&x_set);
 	wS = asInteger(wordSize); // [1 to 20]
 	maskReps = asInteger(mask);
+	double prob = pow(0.7, wS); // expected probability of match
+	double missed = -48.35429; // log(probability of no matches)
+	int maxMismatches = (int)(missed/log(1 - prob)); // interval between matches
 	
 	SEXP ret_list;
 	PROTECT(ret_list = allocVector(VECSXP, x_length));
@@ -659,16 +887,52 @@ SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask,
 			m = *(alpha + i);
 	}
 	
-	int SIZE[20] = {0, 60, 38, 30, 26, 24, 22, 20, 19, 19, 18, 17, 17, 16, 16, 15, 15, 15, 15, 14};
+	// initialize variables for masking low complexity regions
+	int WINDOWSIZE[20] = {0, 24, 22, 20, 18, 17, 15, 14, 13, 12, 12, 11, 11, 10, 10, 9, 9, 9, 9, 8};
+	int maskSimp = (asInteger(maskLCRs) == 0) ? 0 : WINDOWSIZE[m]; // window size
+	double THRESHOLD[20] = {0, 18.189, 21.64, 24.462, 26.987, 29.327, 31.539, 33.653, 35.691, 37.667, 39.591, 41.47, 43.311, 45.118, 46.895, 48.645, 50.372, 52.076, 53.76, 55.426};
+	double threshold = THRESHOLD[m];
+	int WINDOWSIZE2[20] = {0, 56, 54, 54, 53, 52, 51, 51, 50, 50, 50, 49, 49, 49, 48, 48, 48, 48, 48, 47};
+	int maskSimp2 = (asInteger(maskLCRs) == 0) ? 0 : WINDOWSIZE2[m]; // window size
+	double THRESHOLD2[20] = {0, 73.513, 78.288, 82.27, 85.853, 89.178, 92.317, 95.311, 98.19, 100.973, 103.674, 106.304, 108.872, 111.385, 113.848, 116.267, 118.646, 120.987, 123.294, 125.57};
+	double threshold2 = THRESHOLD2[m];
+	
+	// initialize variables for masking repeat regions
+	int SIZE[20] = {0, 54, 34, 27, 23, 21, 19, 18, 17, 16, 16, 15, 15, 14, 14, 14, 13, 13, 13, 13};
 	int n = SIZE[m]; // minimum length of significant repeat
 	m++; // start at 1
 	
+	// initialize variables for masking numerous k-mers
+	int l = length(maskNum);
+	int *mN;
+	int tot; // total number of k-mers
+	if (l > 0) {
+		mN = INTEGER(maskNum);
+		tot = 1;
+		for (i = 0; i < wS; i++)
+			tot *= m;
+	}
+	
+	double E[m]; // expected frequency
+	if (maskSimp) { // assume uniform frequency distribution
+		for (i = 0; i < m; i++)
+			E[i] = 0;
+		for (i = 0; i < 20; i++)
+			E[*(alpha + i)]++;
+		for (i = 0; i < m; i++)
+			E[i] /= 20;
+	}
+	
 	// fill the position weight vector
-	int pwv[wS]; // wS[0] is ignored
-	if (wS > 1)
-		pwv[1] = m;
-	for (i = 2; i < wS; i++) {
-		pwv[i] = pwv[i - 1]*m;
+	int pwv[wS];
+	if (fast) { // left side moves faster
+		pwv[0] = 1;
+		for (i = 1; i < wS; i++)
+			pwv[i] = pwv[i - 1]*m;
+	} else { // right side moves faster
+		pwv[wS - 1] = 1;
+		for (i = wS - 2; i >= 0; i--)
+			pwv[i] = pwv[i + 1]*m;
 	}
 	
 	// build a vector of thread-safe pointers
@@ -687,7 +951,9 @@ SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask,
 		UNPROTECT(1);
 	}
 	
+	#ifdef _OPENMP
 	#pragma omp parallel for private(i,j,k,x_i,rans,sum,ambiguous) num_threads(nthreads)
+	#endif
 	for (i = 0; i < x_length; i++) {
 		x_i = get_elt_from_XStringSet_holder(&x_set, i);
 		if ((x_i.length - wS + 1) >= 1) {
@@ -698,7 +964,7 @@ SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask,
 			}
 			for (j = wS - 1; j < x_i.length; j++) {
 				alphabetFrequencyReducedAA(&x_i, &bases[wS - 1], j, alpha);
-				sum = bases[0];
+				sum = bases[0]*pwv[0];;
 				ambiguous = 0;
 				if (bases[0] < 0)
 					ambiguous = 1;
@@ -716,7 +982,15 @@ SEXP enumerateSequenceReducedAA(SEXP x, SEXP wordSize, SEXP alphabet, SEXP mask,
 			}
 			
 			if (maskReps)
-				maskRepeats(rans, wS, 3, 11, n, x_i.length - wS + 1);
+				maskRepeats(rans, wS, 1, 500, n, maxMismatches, x_i.length - wS + 1, prob);
+			
+			if (maskSimp)
+				maskSimple(rans, wS, E, m, maskSimp, threshold, x_i.length - wS + 1);
+			if (maskSimp2)
+				maskSimple(rans, wS, E, m, maskSimp2, threshold2, x_i.length - wS + 1);
+			
+			if (l == x_length)
+				maskNumerous(rans, mN[i], tot, x_i.length - wS + 1, wS);
 		}
 	}
 	
