@@ -55,9 +55,9 @@ static const R_CallMethodDef callMethods[] = { // method call, pointer, num args
 	{"consensusProfile", (DL_FUNC) &consensusProfile, 3},
 	{"consensusProfileAA", (DL_FUNC) &consensusProfileAA, 3},
 	{"adjustHeights", (DL_FUNC) &adjustHeights, 1},
-	{"enumerateSequence", (DL_FUNC) &enumerateSequence, 4},
+	{"enumerateSequence", (DL_FUNC) &enumerateSequence, 7},
 	{"enumerateSequenceAA", (DL_FUNC) &enumerateSequenceAA, 2},
-	{"enumerateSequenceReducedAA", (DL_FUNC) &enumerateSequenceReducedAA, 5},
+	{"enumerateSequenceReducedAA", (DL_FUNC) &enumerateSequenceReducedAA, 8},
 	{"enumerateGappedSequence", (DL_FUNC) &enumerateGappedSequence, 3},
 	{"enumerateGappedSequenceAA", (DL_FUNC) &enumerateGappedSequenceAA, 3},
 	{"matchOrder", (DL_FUNC) &matchOrder, 4},
@@ -82,14 +82,13 @@ static const R_CallMethodDef callMethods[] = { // method call, pointer, num args
 	{"composition", (DL_FUNC) &composition, 1},
 	{"matchListsDual", (DL_FUNC) &matchListsDual, 5},
 	{"findFrameshifts", (DL_FUNC) &findFrameshifts, 14},
-	{"radixOrder", (DL_FUNC) &radixOrder, 3},
+	{"radixOrder", (DL_FUNC) &radixOrder, 5},
 	{"fillOverlaps", (DL_FUNC) &fillOverlaps, 2},
 	{"indexByContig", (DL_FUNC) &indexByContig, 5},
-	{"chainSegments", (DL_FUNC) &chainSegments, 19},
+	{"chainSegments", (DL_FUNC) &chainSegments, 21},
 	{"basicTranslate", (DL_FUNC) &basicTranslate, 3},
 	{"firstSeqsGapsEqual", (DL_FUNC) &firstSeqsGapsEqual, 9},
 	{"positionWeightMatrix", (DL_FUNC) &positionWeightMatrix, 4},
-	{"extendSegments", (DL_FUNC) &extendSegments, 13},
 	{"firstSeqsPosEqual", (DL_FUNC) &firstSeqsPosEqual, 9},
 	{"collapse", (DL_FUNC) &collapse, 3},
 	{"nbit", (DL_FUNC) &nbit, 4},
@@ -148,21 +147,24 @@ static const R_CallMethodDef callMethods[] = { // method call, pointer, num args
 	{"dist", (DL_FUNC) &dist, 2},
 	{"clusterMP", (DL_FUNC) &clusterMP, 8},
 	{"expM", (DL_FUNC) &expM, 3},
-	{"matchOverlap", (DL_FUNC) &matchOverlap, 5},
-	{"similarities", (DL_FUNC) &similarities, 8},
 	{"alignPair", (DL_FUNC) &alignPair, 13},
 	{"overlap", (DL_FUNC) &overlap, 3},
 	{"cophenetic", (DL_FUNC) &cophenetic, 4},
-	{"countHits", (DL_FUNC) &countHits, 3},
 	{"sumBins", (DL_FUNC) &sumBins, 2},
 	{"dereplicate", (DL_FUNC) &dereplicate, 2},
-	{"selectIndices", (DL_FUNC) &selectIndices, 6},
-	{"selectGroups", (DL_FUNC) &selectGroups, 4},
+	{"selectGroups", (DL_FUNC) &selectGroups, 5},
 	{"firstHit", (DL_FUNC) &firstHit, 2},
 	{"xorShift", (DL_FUNC) &xorShift, 2},
-	{"maxGroup", (DL_FUNC) &maxGroup, 2},
 	{"sortedUnique", (DL_FUNC) &sortedUnique, 1},
 	{"splitPartitions", (DL_FUNC) &splitPartitions, 5},
+	{"searchIndex", (DL_FUNC) &searchIndex, 23},
+	{"detectCores", (DL_FUNC) &detectCores, 0},
+	{"countIndex", (DL_FUNC) &countIndex, 3},
+	{"updateIndex", (DL_FUNC) &updateIndex, 8},
+	{"approxFreqs", (DL_FUNC) &approxFreqs, 3},
+	{"alignPairs", (DL_FUNC) &alignPairs, 15},
+	{"extendMatches", (DL_FUNC) &extendMatches, 14},
+	{"computeOverlap", (DL_FUNC) &computeOverlap, 18},
 	{NULL, NULL, 0}
 };
 
